@@ -1,55 +1,50 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000 - 2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
-#import <stdio.h>
-#import <unistd.h>
-#import <stdlib.h>
-#import <mach/mach.h>
-#import <mach/mach_error.h>
-#import <servers/bootstrap.h>
-#import <syslog.h>
-#import <CoreFoundation/CFMachPort.h>
+#include <stdio.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <mach/mach.h>
+#include <mach/mach_error.h>
+#include <servers/bootstrap.h>
+#include <syslog.h>
+#include <CoreFoundation/CFMachPort.h>
 
-#import "dprintf.h"
-#import "threadcompat.h"
-#import "machcompat.h"
-#import "ipconfig.h"
-#import "ipconfigd.h"
-#import "ipconfig_ext.h"
-#import "globals.h"
-
-//static char request_buf[1024];
-static char reply_buf[1024];
+#include "dprintf.h"
+#include "ipconfig.h"
+#include "ipconfigd.h"
+#include "ipconfig_ext.h"
+#include "globals.h"
 
 static uid_t S_uid = -1;
 static gid_t S_gid = -1;
 
-#ifdef MOSX
 static __inline__ void
-read_trailer(msg_header_t * request)
+read_trailer(mig_reply_error_t * request)
 {
     mach_msg_format_0_trailer_t	*trailer;
     trailer = (mach_msg_security_trailer_t *)((vm_offset_t)request +
-					      round_msg(request->msgh_size));
+					      round_msg(request->Head.msgh_size));
 
     if ((trailer->msgh_trailer_type == MACH_MSG_TRAILER_FORMAT_0) &&
 	(trailer->msgh_trailer_size >= MACH_MSG_TRAILER_FORMAT_0_SIZE)) {
@@ -57,20 +52,10 @@ read_trailer(msg_header_t * request)
 	S_gid = trailer->msgh_sender.val[1];
     }
     else {
-	/* XXX
-	 * Change 0 to -1 in the following two lines when CF is fixed.
-	 */
-	S_uid = 0;
-	S_gid = 0;
+	S_uid = -1;
+	S_gid = -1;
     }
 }
-#else MOSX
-static __inline__ void
-read_trailer(msg_header_t * request)
-{
-    return;
-}
-#endif MOSX
 
 kern_return_t
 _ipconfig_config_if(port_t p, if_name_t name)
@@ -173,56 +158,27 @@ _ipconfig_set(port_t p, if_name_t name,
     return (KERN_SUCCESS);
 }
 
-#if 0
-void 
-server_loop(void * arg)
+kern_return_t
+_ipconfig_set_verbose(port_t p, int verbose,
+		      ipconfig_status_t * status)
 {
-    msg_header_t *	request = (msg_header_t *)request_buf;
-    msg_header_t *	reply = (msg_header_t *)reply_buf;
-
-    while (1) {
-	msg_return_t r;
-#ifdef MOSX
-	request->msgh_size = sizeof(request_buf);
-	request->msgh_local_port = service_port;
-	r = msg_receive(request, 
-			MSG_OPTION_NONE 
-			| MACH_RCV_TRAILER_ELEMENTS(MACH_RCV_TRAILER_SENDER)
-			| MACH_RCV_TRAILER_TYPE(MACH_MSG_TRAILER_FORMAT_0), 
-			0);
-#else MOSX
-	request->msg_size = sizeof(request_buf);
-	request->msg_local_port = service_port;
-	r = msg_receive(request, MSG_OPTION_NONE, 0);
-#endif
-	if (r == RCV_SUCCESS) {
-	    extern boolean_t ipconfig_server(msg_header_t *, msg_header_t *);
-
-	    read_trailer(request);
-	    if (ipconfig_server(request, reply)) {
-		r = msg_send(reply, MSG_OPTION_NONE, 0);
-		if (r != SEND_SUCCESS)
-		    my_log(LOG_INFO, "msg_send: %s", mach_error_string(r));
-	    }
-	}
-	else {
-#ifdef MOSX	    
-	    my_log(LOG_INFO, "msg_receive: %s (0x%x)", 
-		   mach_error_string(r), r);
-	    if (r == MACH_RCV_INVALID_NAME) {
-		break; /* out of while */
-	    }
-#else MOSX
-	    my_log(LOG_INFO, "msg_receive: %s (%d)", mach_error_string(r), r);
-	    if (r == RCV_INVALID_PORT) {
-		break; /* out of while */
-	    }
-#endif MOSX
-	}
+    if (S_uid != 0) {
+	*status = ipconfig_status_permission_denied_e;
     }
+    else {
+	*status = set_verbose(verbose);
+    }
+    return (KERN_SUCCESS);
 }
 
-#endif 0
+#ifdef IPCONFIG_TEST_NO_ENTRY
+kern_return_t
+_ipconfig_set_something(port_t p, int verbose,
+			ipconfig_status_t * status)
+{
+    return (KERN_SUCCESS);
+}
+#endif IPCONFIG_TEST_NO_ENTRY
 
 boolean_t
 server_active()
@@ -237,21 +193,63 @@ server_active()
     return (FALSE);
 }
 
-static CFMachPortRef	ipconfigd_port;
-
 static void
 S_ipconfig_server(CFMachPortRef port, void *msg, CFIndex size, void *info)
 {
-    msg_header_t *	request = (msg_header_t *)msg;
-    msg_header_t *	reply = (msg_header_t *)reply_buf;
-    msg_return_t 	r;
-    extern boolean_t 	ipconfig_server(msg_header_t *, msg_header_t *);
+    char 		reply_buf[1024];
+    mach_msg_options_t 	options = 0;
+    mig_reply_error_t * request = (mig_reply_error_t *)msg;
+    mig_reply_error_t *	reply = (mig_reply_error_t *)reply_buf;
+    mach_msg_return_t 	r = MACH_MSG_SUCCESS;
+    extern boolean_t 	ipconfig_server(mig_reply_error_t *, 
+					mig_reply_error_t *);
 
     read_trailer(request);
-    if (ipconfig_server(request, reply)) {
-	r = msg_send(reply, MSG_OPTION_NONE, 0);
-	if (r != SEND_SUCCESS)
-	    my_log(LOG_INFO, "msg_send: %s", mach_error_string(r));
+    if (ipconfig_server(request, reply) == FALSE) {
+	my_log(LOG_INFO, "IPConfiguration: unknown message ID (%d) received",
+	       request->Head.msgh_id);
+    }
+
+    /* Copied from Libc/mach/mach_msg.c:mach_msg_server_once(): Start */
+    if (!(reply->Head.msgh_bits & MACH_MSGH_BITS_COMPLEX)) {
+	if (reply->RetCode == MIG_NO_REPLY)
+	    reply->Head.msgh_remote_port = MACH_PORT_NULL;
+	else if ((reply->RetCode != KERN_SUCCESS) &&
+		 (request->Head.msgh_bits & MACH_MSGH_BITS_COMPLEX)) {
+	    /* destroy the request - but not the reply port */
+	    request->Head.msgh_remote_port = MACH_PORT_NULL;
+	    mach_msg_destroy(&request->Head);
+	}
+    }
+    /*
+     *	We don't want to block indefinitely because the client
+     *	isn't receiving messages from the reply port.
+     *	If we have a send-once right for the reply port, then
+     *	this isn't a concern because the send won't block.
+     *	If we have a send right, we need to use MACH_SEND_TIMEOUT.
+     *	To avoid falling off the kernel's fast RPC path unnecessarily,
+     *	we only supply MACH_SEND_TIMEOUT when absolutely necessary.
+     */
+    if (reply->Head.msgh_remote_port != MACH_PORT_NULL) {
+	r = mach_msg(&reply->Head,
+		     (MACH_MSGH_BITS_REMOTE(reply->Head.msgh_bits) ==
+		      MACH_MSG_TYPE_MOVE_SEND_ONCE) ?
+		     MACH_SEND_MSG|options :
+		     MACH_SEND_MSG|MACH_SEND_TIMEOUT|options,
+		     reply->Head.msgh_size, 0, MACH_PORT_NULL,
+		     MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
+	if ((r != MACH_SEND_INVALID_DEST) &&
+	    (r != MACH_SEND_TIMED_OUT))
+	    goto done_once;
+	r = MACH_MSG_SUCCESS;
+    }
+    if (reply->Head.msgh_bits & MACH_MSGH_BITS_COMPLEX)
+	mach_msg_destroy(&reply->Head);
+ done_once:
+    /* Copied from Libc/mach/mach_msg.c:mach_msg_server_once(): End */
+
+    if (r != MACH_MSG_SUCCESS) {
+	my_log(LOG_INFO, "IPConfiguration msg_send: %s", mach_error_string(r));
     }
     return;
 }
@@ -259,8 +257,9 @@ S_ipconfig_server(CFMachPortRef port, void *msg, CFIndex size, void *info)
 void
 server_init()
 {
-    CFRunLoopSourceRef	rls;
     boolean_t		active;
+    CFRunLoopSourceRef	rls;
+    CFMachPortRef	ipconfigd_port;
     kern_return_t 	status;
 
     active = FALSE;
