@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -84,6 +85,7 @@
 #include <mach/boolean.h>
 #include <signal.h>
 #include <stdio.h>
+#include <stdarg.h>
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
@@ -119,9 +121,6 @@
 #define CFGPROP_DENY			"deny"
 #define CFGPROP_REPLY_THRESHOLD_SECONDS	"reply_threshold_seconds"
 
-/* external functions */
-extern struct ether_addr *	ether_aton(char *);
-
 /* local defines */
 #define	MAXIDLE			(5*60)	/* we hang around for five minutes */
 #define DOMAIN_HIERARCHY	"..."	/* ... means open the hierarchy */
@@ -187,7 +186,10 @@ my_log(int priority, const char *message, ...)
     if (priority == LOG_DEBUG) {
 	if (verbose == FALSE)
 	    return;
-	priority = LOG_INFO;
+	priority = LOG_NOTICE;
+    }
+    else if (priority == LOG_INFO) {
+	priority = LOG_NOTICE;
     }
     if (quiet && (priority > LOG_ERR)) { 
 	return;
@@ -844,14 +846,14 @@ S_update_services()
     reply_threshold_seconds = 0;
     nl_p = PropList_lookup(&S_config_dhcp, CFGPROP_REPLY_THRESHOLD_SECONDS);
     if (nl_p != NULL && nl_p->ninl_len != 0) {
-	reply_threshold_seconds = strtoul(nl_p->ninl_val[0], NULL, NULL);
+	reply_threshold_seconds = strtoul(nl_p->ninl_val[0], NULL, 0);
     }
 
     /* detect other DHCP server */
     detect_other_dhcp_server = 0;
     nl_p = PropList_lookup(&S_config_dhcp, CFGPROP_DETECT_OTHER_DHCP_SERVER);
     if (nl_p != NULL && nl_p->ninl_len != 0) {
-	if (strtol(nl_p->ninl_val[0], NULL, NULL) != 0) {
+	if (strtol(nl_p->ninl_val[0], NULL, 0) != 0) {
 	    detect_other_dhcp_server = 1;
 	}
     }
@@ -893,7 +895,7 @@ old_netboot_enabled(interface_t * if_p)
 void
 usage()
 {
-    fprintf(stderr, "useage: bootpd <options>\n"
+    fprintf(stderr, "usage: bootpd <options>\n"
 	    "<options> are:\n"
 	    "[ -a ] 	support anonymous binding for BOOTP clients\n"
 	    "[ -D ]	be a DHCP server\n"
@@ -947,7 +949,7 @@ main(int argc, char * argv[])
 	    /* reply only if bootfile exists */
 	    break;
 	case 'c':		    /* cache check interval - seconds */
-	    S_cache_check_interval = strtoul(optarg, NULL, NULL);
+	    S_cache_check_interval = strtoul(optarg, NULL, 0);
 	    printf("Using cache check interval %ld seconds\n",
 		   S_cache_check_interval);
 	    break;
@@ -1007,7 +1009,7 @@ main(int argc, char * argv[])
 	    break;
 	}
 	case 'p': {
-	    server_priority = strtoul(optarg, NULL, NULL);
+	    server_priority = strtoul(optarg, NULL, 0);
 	    printf("Priority set to %d\n", server_priority);
 	    break;
 	}
@@ -1252,8 +1254,8 @@ on_alarm(int sigraised)
  *   get its own version of the bootfile.
  */
 boolean_t
-bootp_add_bootfile(char * request_file, char * hostname,
-		   char * bootfile, char * reply_file)
+bootp_add_bootfile(const char * request_file, const char * hostname,
+		   const char * bootfile, char * reply_file)
 {
     boolean_t 	dothost = FALSE;	/* file.host was found */
     char 	file[PATH_MAX];
@@ -1407,7 +1409,7 @@ bootp_request(request_t * request)
     u_char *		hostname = NULL;
     struct in_addr	iaddr;
     boolean_t		netinfo_host = FALSE;
-    static u_char	netinfo_options[] = {
+    static const u_char	netinfo_options[] = {
 	dhcptag_subnet_mask_e, 
 	dhcptag_router_e, 
 	dhcptag_netinfo_server_address_e,
@@ -1416,8 +1418,7 @@ bootp_request(request_t * request)
 	dhcptag_domain_name_e,
 	dhcptag_host_name_e,
     };
-    static int		n_netinfo_options 
-	= sizeof(netinfo_options) / sizeof(netinfo_options[0]);
+#define N_NETINFO_OPTIONS (sizeof(netinfo_options) / sizeof(netinfo_options[0]))
     struct bootp 	rp;
     struct bootp *	rq = (struct bootp *)request->pkt;
     u_int16_t		secs;
@@ -1488,7 +1489,7 @@ bootp_request(request_t * request)
 	    my_log(LOG_DEBUG, "netinfo client");
 	    add_subnet_options(domain, hostname, iaddr, 
 			       request->if_p, &options, 
-			       netinfo_options, n_netinfo_options);
+			       netinfo_options, N_NETINFO_OPTIONS);
 	}
 	else {
 	    add_subnet_options(domain, hostname, iaddr, 
@@ -1596,7 +1597,7 @@ get_dhcp_option(id subnet, int tag, void * buf, int * len_p)
     unsigned char	err[256];
     unsigned char	propname[128];
     ni_namelist	*	nl_p;
-    unsigned char *	tag_name;
+    const char *	tag_name;
 
     tag_name = dhcptag_name(tag);
     if (tag_name == NULL)
@@ -1633,28 +1634,27 @@ get_dhcp_option(id subnet, int tag, void * buf, int * len_p)
 int
 add_subnet_options(NIDomain_t * domain, u_char * hostname, 
 		   struct in_addr iaddr, interface_t * if_p,
-		   dhcpoa_t * options, u_char * tags, int n)
+		   dhcpoa_t * options, const u_char * tags, int n)
 {
     inet_addrinfo_t *	info = if_inet_addr_at(if_p, 0);
     char		buf[DHCP_OPTION_SIZE_MAX];
     int			len;
-    static u_char 	default_tags[] = { 
+    static const u_char 	default_tags[] = { 
 	dhcptag_subnet_mask_e, 
 	dhcptag_router_e, 
 	dhcptag_domain_name_server_e,
 	dhcptag_domain_name_e,
 	dhcptag_host_name_e,
     };
+#define N_DEFAULT_TAGS	(sizeof(default_tags) / sizeof(default_tags[0]))
     boolean_t		netinfo_done = FALSE;
-    static int		n_default_tags 
-	= sizeof(default_tags) / sizeof(default_tags[0]);
     int			number_before = dhcpoa_count(options);
     int			i;
     id			subnet = [subnets entry:iaddr];
 
     if (tags == NULL) {
-	tags = default_tags;
-	n = n_default_tags;
+	tags = (u_char *)default_tags;
+	n = N_DEFAULT_TAGS;
     }
 			
     for (i = 0; i < n; i++ ) {
@@ -1808,7 +1808,7 @@ add_subnet_options(NIDomain_t * domain, u_char * hostname,
 /**
  ** Server Main Loop
  **/
-static char 		control[1024];
+static char 		control[512];
 static struct iovec  	iov;
 static struct msghdr 	msg;
 
@@ -1974,6 +1974,9 @@ S_dispatch_packet(struct bootp * bp, int n, interface_t * if_p,
 				     is_old_netboot);
 		    }
 		}
+		else {
+		    bsdp_dhcp_request(&request, dhcp_msgtype);
+		}
 		dhcpol_free(&rq_vsopt);
 	    }
 	    if (dhcp_enabled(if_p) || old_netboot_enabled(if_p)) {
@@ -2260,13 +2263,12 @@ S_relay_loop(struct in_addr * relay, int max_hops)
 
 #include <SystemConfiguration/SystemConfiguration.h>
 #include <SystemConfiguration/SCDynamicStorePrivate.h>
-
+static SCDynamicStoreRef	store;
 static void
 S_add_ip_change_notifications()
 {
     CFStringRef			key;
     CFMutableArrayRef		patterns;
-    SCDynamicStoreRef		store;
 
     store = SCDynamicStoreCreate(NULL,
 				 CFSTR("com.apple.network.bootpd"),
